<?php

declare(strict_types=1);

namespace RebelCode\Spotlight\Instagram\Engine;

use RebelCode\Iris\Engine;
use RebelCode\Iris\Fetcher\FetchQuery;
use RebelCode\Iris\Fetcher\FetchResult;
use RebelCode\Spotlight\Instagram\Wp\CronJob;

class Importer
{
    /** @var Engine */
    protected $engine;

    /** @var int */
    protected $batchSize;

    /** @var int */
    protected $batchDelay;

    /** @var string */
    protected $batchCronHook;

    public function __construct(Engine $engine, int $batchSize, int $batchDelay, string $batchCronHook)
    {
        $this->engine = $engine;
        $this->batchSize = $batchSize;
        $this->batchDelay = $batchDelay;
        $this->batchCronHook = $batchCronHook;
    }

    public function updateSources(array $sources): array
    {
        $items = [];
        $errors = [];
        $scheduledBatch = false;

        foreach ($sources as $source) {
            /** @var $subResult FetchResult */
            /** @var $subScheduledBatch bool */
            [$subResult, $subScheduledBatch] = $this->importBatch(new FetchQuery($source, null, $this->batchSize));

            $items = array_merge($items, $subResult->items);
            $errors = array_merge($errors, $subResult->errors);
            $scheduledBatch = $scheduledBatch || $subScheduledBatch;
        }

        return [$items, $scheduledBatch, $errors];
    }

    public function importBatch(FetchQuery $query): array
    {
        // Allocate up to 30 minutes for each batch
        set_time_limit(30 * 60);

        $result = $this->engine->import($query);

        $nextQuery = $query->forNextBatch($result);
        $scheduledBatch = false;
        if ($nextQuery !== null) {
            $job = new CronJob($this->batchCronHook, [$nextQuery]);
            CronJob::schedule($job, time() + $this->batchDelay);
            $scheduledBatch = true;
        }

        return [$result, $scheduledBatch];
    }
}
